function [RVM_array] = generate_RVM_signal2(modulation_scheme,array_bit_length,preamble_bits, data_bits, signal_start_index, signal_amplitude, snr, samples_per_bit,periods_per_bit,signal_scew)
%GENERATE_RVM_SIGNAL Summary of this function goes here
%   Detailed explanation goes here
%   ModulationScheme = ASK, PSK, FSK etc
%   preamble = The actual preamble (standard Bluetooth: 1 byte 1010101010,
%   ethernet: 8 bytes, 7 with 10101010, last has 10101011)
%   SoF_delimiter = how the start of frame delimiter looks
%       Start of Frame Byte: 10101011
%   data = the data part of the package
%   startIndex = From which sample/bit start position should the packet start
%       outputLength = length of output array, measured as number of bits. So
%       for 4 samples per bit and outputLength of 100 bits, the array has the
%       size of 400
%   samplesPerBiy = number of samples used for the duration of 1 bit.
%   signalStrength = Amplitude of signal measured from 0-1 times the legal
%   values.
%   noiseStrength = Maximum amplitude allowed for noise values, between 0-1
%   times of the legal value. NB: If signalStrength + noiseStrength >1, one
%   can reach overflow

%generate_RVM_signal("ASK",100, [1 0 1 0 1 0 1 1], [0 0 1 1 0 0 0 1 1 1 0 1 0 1], 1, 0.5, 10, 10, 1, 0)


%Construct the actual signal
signal = [];
signal = horzcat(signal, preamble_bits, data_bits);
bit_shift = 1;
if signal_start_index == array_bit_length
    bit_shift = 0;
end


%Construct entire signal spectrum
bit_signal = ones(1,array_bit_length)*3;



%signal_start_index = 5
elements_to_copy = length(signal);
if (signal_start_index + elements_to_copy - 1) > length(bit_signal)
    elements_to_copy = length(bit_signal) - signal_start_index + 1;
end

signal_end_index = signal_start_index + elements_to_copy -1;


bit_signal(signal_start_index:signal_end_index) = signal(1:elements_to_copy);








amplitude_b1 = 100.0*signal_amplitude;
amplitude_b0 = 0;
psk_difference = pi;
fsk_multiplier = 2;
scew = signal_scew*pi;
bit_sample_progress = 0:1/samples_per_bit:(1-(1/samples_per_bit));
output_array = [];





if modulation_scheme == "ASK"
    %disp("Modulation Scheme: ASK")
    for (i=1:1:length(bit_signal))
        if (bit_signal(i)==1)
        y=amplitude_b1*cos(scew+2*pi*periods_per_bit*bit_sample_progress);
        elseif (bit_signal(i)==0)
        y=amplitude_b0*cos(scew+2*pi*periods_per_bit*bit_sample_progress);
        else
        y=0*cos(scew+2*pi*periods_per_bit*bit_sample_progress);
        end
        output_array=[output_array y];
    end
elseif modulation_scheme == "PSK"
    %PSK here
    for (i=1:1:length(bit_signal))
        if (bit_signal(i)==1)
        y=amplitude_b1*cos(scew+2*pi*periods_per_bit*bit_sample_progress);
        elseif (bit_signal(i)==0)
        y=amplitude_b1*cos(scew+2*pi*periods_per_bit*bit_sample_progress+psk_difference);
        else
        y=0*cos(scew+2*pi*periods_per_bit*bit_sample_progress);
        end
        output_array=[output_array y];
    end
elseif modulation_scheme == "FSK"
    %FSK here
    for (i=1:1:length(bit_signal))
        if (bit_signal(i)==1)
        y=amplitude_b1*cos(scew+2*pi*periods_per_bit*bit_sample_progress);
        elseif (bit_signal(i)==0)
        y=amplitude_b1*cos(scew+2*pi*periods_per_bit*bit_sample_progress*fsk_multiplier);
        else
        y=0*cos(scew+2*pi*periods_per_bit*bit_sample_progress);
        end
        output_array=[output_array y];
    end
end

%Move samples (0, samples-per-bit -1) to the right)
if bit_shift == 1
    shift_number = randi([1, samples_per_bit-1]);
    output_array(1+shift_number: length(output_array)) = output_array(1:length(output_array)-shift_number);
    output_array(1:shift_number) = 0;
end



%Change the last number below to alter actual SNR.
%Value of 25 gives average value around 12.5 and max value around 50 for
%SNR = 1. The value represents the signalpower used instead of in
output_array = awgn(output_array,snr,25);

plot(output_array);



RVM_array = output_array;

end


